

from .functions import (_drain, _on_finished, start, stop)

def initFuncs(self):
    try:
        for f in (_drain, _on_finished, start, stop):
            setattr(self, f.__name__, f)
    except Exception as e:
        logger.info(f"{e}")
    return self



import os
from .startConsole import startConsole
from .initFuncsCall import getInitForAllTabs
ABS_PATH = os.path.abspath(__file__)
ABS_DIR = os.path.dirname(ABS_PATH)
getInitForAllTabs(ABS_DIR)
from .windowManagerConsole import windowManagerConsole,startWindowManagerConsole
from .appRunnerTab import appRunnerTab,startAppRunnerConsole
from .launcherWindowTab import launcherWindowTab,startLauncherWindowConsole
from .logPaneTab import logPaneTab,startLogPaneTabConsole




# table_factory_headers_config.py
from typing import Any, Dict, Iterable, List, Optional, Sequence, Tuple
from PyQt6.QtWidgets import QTableWidget, QTableWidgetItem, QVBoxLayout, QLabel, QWidget, QAbstractItemView, QHeaderView
from PyQt6.QtCore import Qt
from abstract_utilities import is_number
# --- small utility: enum resolver by name or pass-through ---
def _enum_from_name(enum_cls, name_or_enum):
    if isinstance(name_or_enum, enum_cls):
        return name_or_enum
    if name_or_enum is None:
        return None
    key = str(name_or_enum).replace(" ", "").replace("-", "").replace("_", "").lower()
    for m in enum_cls:  # iterate members
        if m.name.replace("_", "").lower() == key:
            return m
    raise ValueError(f"Unknown {enum_cls.__name__} member: {name_or_enum!r}")

def _normalize_headers_and_data(
    data: Optional[Iterable],
    headers: Optional[Sequence[str]]
) -> Tuple[List[str], List[List[str]]]:
    data = list(data or [])
    if not data:
        return list(headers or []), []
    if isinstance(data[0], dict):
        if not headers:
            first = list(data[0].keys())
            seen = set(first)
            extra = []
            for row in data[1:]:
                for k in row.keys():
                    if k not in seen:
                        extra.append(k); seen.add(k)
            headers = first + extra
        rows = [[str(d.get(h, "")) for h in headers] for d in data]
        return list(headers), rows
    # list/tuple rows
    if not headers:
        width = max(len(r) for r in data)
        headers = [f"Col {i}" for i in range(width)]
    width = len(headers)
    rows = [[str(r[i]) if i < len(r) else "" for i in range(width)] for r in data]
    return list(headers), rows

def set_table_data(table: QTableWidget, data: Iterable, headers: Optional[Sequence[str]] = None):
    headers, rows = _normalize_headers_and_data(data, headers)
    table.blockSignals(True)
    table.clear()
    table.setColumnCount(len(headers))
    if headers:
        table.setHorizontalHeaderLabels(headers)
    table.setRowCount(len(rows))
    for r, row in enumerate(rows):
        for c, text in enumerate(row):
            table.setItem(r, c, QTableWidgetItem(text))
    table.blockSignals(False)





#!/usr/bin/env python3
from typing import *
from functools import partial
from abstract_utilities import eatAll
from abstract_utilities.type_utils import MIME_TYPES
from PyQt6.QtGui import QTextCursor, QColor, QBrush # add this import
from PyQt6.QtCore import (
    Qt, QObject, QThread, pyqtSignal, qInstallMessageHandler,
    QtMsgType, QTimer, QProcess
)
from PyQt6.QtWidgets import (
    QLabel, QGridLayout, QPushButton, QVBoxLayout, QHBoxLayout,
    QCompleter, QFileDialog, QHeaderView, QLineEdit, QWidget,
    QListView, QComboBox, QFormLayout, QAbstractItemView, QCheckBox,
    QMainWindow, QTableWidgetItem, QTableWidget, QTextEdit,
    QApplication, QSizePolicy, QStatusBar, QMessageBox, QTabWidget,
    QListWidget, QButtonGroup,QRadioButton,QListWidget,QListWidgetItem
    )
import time, re, pydot, inspect, threading, enum, sys, requests, subprocess, os, logging, traceback, re, json
from abstract_utilities import is_number,make_list,safe_read_from_json,read_from_file,make_dirs
from functools import partial, lru_cache



from .functions import *
from .widgets import *
from .imports import *
from .factories import *
from .getFnames import getInitForAllTabs
from .managers import *
from .startConsole import startConsole
